# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function, division, absolute_import

import hashlib
import os
import random
import time

from django.utils.deconstruct import deconstructible
from django.utils.text import slugify


@deconstructible
class UploadToDir(object):
    """Generates a function to give  to ``upload_to`` parameter in
    models.Fields, that generates an name for uploaded files based on ``populate_from``
    attribute.
    """

    def __init__(self, path, populate_from=None, prefix=None, random_name=False):
        self.path = path
        self.populate_from = populate_from
        self.random_name = random_name
        self.prefix = prefix

    def __call__(self, instance, filename):
        """Generates an name for an uploaded file."""
        if self.populate_from is not None and not hasattr(instance, self.populate_from):
            raise AttributeError("Instance hasn't {} attribute".format(self.populate_from))
        ext = filename.split('.')[-1]
        readable_name = slugify(filename.split('.')[0])
        if self.populate_from:
            readable_name = slugify(getattr(instance, self.populate_from))
        if self.random_name:
            random_name = hashlib.sha256(
                "{}--{}".format(time.time(), random.random()).encode('utf-8')
            )
            readable_name = random_name.hexdigest()
        elif self.prefix is not None:
            readable_name = f'{self.prefix}{readable_name}'
        file_name = "{}.{}".format(readable_name, ext)
        return os.path.join(self.path, file_name)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function, division, absolute_import

from django.contrib.admin.utils import quote
from django.forms.widgets import Widget
from django.template.defaultfilters import truncatechars
from django.urls import reverse
from django.utils.encoding import smart_text
from django.utils.safestring import mark_safe


class ManyToManyListAdmin(Widget):
    class Media:
        js = ("js/foreign_key_widget.js",)

    @staticmethod
    def _format_item(item):
        url = reverse('admin:%s_%s_change' % (item._meta.app_label, item._meta.model_name),
                      args=(quote(item.pk),), )
        label = truncatechars(smart_text(item), 40)
        return "<a href='{}'>{}</a>".format(url, label)

    def render(self, name, value, attrs=None):
        selected_values = self.choices.queryset.filter(id__in=value)
        return mark_safe(
            "".join(
                ["<ul>"] + map(lambda item: "<li>%s</li>" % self._format_item(item), selected_values) + ["</ul>"]
            )
        )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function, division, absolute_import

from django.contrib.gis.db import models
from django.utils.translation import ugettext_lazy as _
from model_utils.fields import AutoLastModifiedField, AutoCreatedField


class GisTimeStampedModel(models.Model):
    """
    An abstract base class model that provides self-updating
    ``created`` and ``modified`` fields.

    """
    created = AutoCreatedField(_('created'))
    modified = AutoLastModifiedField(_('modified'))

    class Meta:
        abstract = True



from rest_framework import pagination



class StandardPageNumberPagination(pagination.PageNumberPagination):
    """Overwrite to add support to the 'page_size' param."""
    page_size = 10
    page_size_query_param = 'page_size'
    max_page_size = 300



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function, division, absolute_import

import os


class ProgressBarCommand(object):
    """Mixin to add a progress bar to a management command."""

    default_terminal_size = 25
    progress_symbol = "="

    def terminal_size(self):
        """Gets the terminal columns size."""
        try:
            _, columns = os.popen('stty size', 'r').read().split()
            return min(int(columns) - 10, 100)
        except ValueError:
            return self.default_terminal_size

    def bar(self, progress):
        """Shows on the stdout the progress bar for the given progress."""
        if not hasattr(self, "_limit") or not self._limit:
            self._limit = self.terminal_size()
        graph_progress = int(progress * self._limit)
        self.stdout.write('\r', ending='')
        progress_format = "[%-{}s] %d%%".format(self._limit)
        self.stdout.write(
            self.style.SUCCESS(progress_format % (self.progress_symbol * graph_progress, int(progress * 100))),
            ending=''
        )
        self.stdout.flush()



from django.db.utils import IntegrityError
from rest_framework import status
from rest_framework.response import Response
from rest_framework.views import exception_handler


def extra_exception_handler(exc, context):
    # Call REST framework's default exception handler first,
    # to get the standard error response.
    response = exception_handler(exc, context)

    # If response is None, add more cases to the exception handler.
    if response is None:
        if isinstance(exc, IntegrityError):
            data = {"error": str(exc)}
            return Response(data, status=status.HTTP_400_BAD_REQUEST)

    return response
